#####
###  PHPGateway - A Gateway to PHP
##   Copyright (C) 2003	Wei He (hewei@ied.org.cn)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
###  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#####

import sys, os, string
from Globals import DTMLFile, InitializeClass
from PHParser import PHParser
from urllib import quote
from time import gmtime, strftime
import mimetypes

#
# Specify the default pages to be searched and their order when the 
# requested URL is a directory.
#
DEFAULTPAGES = ['index.html', 'index.htm', 'default.htm', 'index.php', 'index.php3']

#
# The extension of scripts that is be regarded as PHP script.
# Other files will be served as static content.
#
PHPEXT = ['.php', '.php3']

#
# The error page.
#
error_html = """
<html>
  <head><title>PHPGateway Error</title></head>
  <body bgcolor="#FFFFFF">
    <h2>Site Error</h2>
    <p>PHPGateway cannot open the destination file.</p> 
    <p><strong>PHPGateway:</strong> <?php echo $HTTP_SERVER_VARS['PHPGATEWAY_SELF']; ?><br />
    <strong>File:</strong> <?php echo $HTTP_SERVER_VARS['REDIRECT_SCRIPT_FILENAME']; ?></p>
    <hr noshade="noshade" />
    <p><?php echo $HTTP_SERVER_VARS['PHPGATEWAY_ERROR_STRING']; ?></p>
    <pre>$ZOPE_VARS=<?php print_r($ZOPE_VARS); ?><br /></pre>
    <?php phpinfo(INFO_ENVIRONMENT); ?>
    </body>
</html>
"""

addPHPGWForm = DTMLFile('dtml/addPHPGWForm', globals())

def addPHPGateway(self, id, title='', docroot='', REQUEST=None, submit=None):
    """Add a PHParser object with the contents of file. If
    'file' is empty, default document text is used.
    """
    id=str(id)
    title=str(title)

    ob = PHPGateway(__name__=id)
    ob.title = title
    ob.isAnObjectManager = 1
    ob.docroot = docroot
    d = self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u + '/' + quote(id) + '/manage_workspace')
    return ''

#
# The class defination of PHPGateway
#
class PHPGateway(PHParser):
    """ PHP Gateway """
    meta_type = 'PHPGateway'

    """ Remove the content tab. I have a better plan on what the first 
        tab should be displaying. :) """
    manage_options = PHParser.manage_options[2:3] + PHParser.manage_options[1:2] + PHParser.manage_options[3:]

    _properties = (
        {'id' : 'title', 'type': 'string', 'mode': 'w'},
        {'id' : 'docroot', 'type' : 'string', 'mode' : 'w'},
    )
    
    #
    # Override the __prepare__ method to specify a fixed PHP script to parse
    #
    def __prepare__(self, src, REQUEST={}):
        cgienv = []

        print "PHPGATEWAY: docroot " + self.docroot

        """ pathsegs will be used to build the real PATH_INFO """
        pathsegs = getattr(self, '__pathsegs__', [])
        print "PHPGATEWAY: pathsegs value: " + str(pathsegs)

        """ locate the script (or static file) in the file system """
        self.__script__ = os.path.join(self.docroot, string.join(pathsegs, os.sep))
        print "PHPGATEWAY: Opening PHP script: " + self.__script__

        failure = ''
        redirection = ''
        pathinfo = REQUEST['PATH_INFO']
        print "PHPGATEWAY: Requested for " + pathinfo

        if os.path.isdir(self.__script__):
            print "PHPGATEWAY: The PHP script is a directory."
            if len(pathinfo) > 0 and pathinfo[-1:] != '/':
                redirection = pathinfo + '/'
                print "PHPGATEWAY: About to redirect to " + redirection
            else:
                print "PHPGATEWAY: Searching for default pages under " + self.__script__
                founddefault = 0
                for page in DEFAULTPAGES:
                    testpage = os.path.join(self.__script__, page)
                    if os.path.isfile(testpage) and not os.path.isdir(testpage):
                        self.__script__ = testpage
                        pathinfo += page
                        print "PHPGATEWAY: Got default page: " + self.__script__
                        founddefault = 1
                        break
                if not founddefault:
                    failure = 'Make sure either of the following file exist: <ul>\n'
                    print "PHPGATEWAY: No default page found."
                    for page in DEFAULTPAGES:
                        testpage = os.path.join(self.__script__, page)
                        failure += '<li>' + testpage + '</li>\n'
                    failure += '</ul>'
                        
        if redirection != '':
            print "PHPGATEWAY: Redirecting to " + redirection
            REQUEST.RESPONSE.redirect(redirection)
            self.__result__ = ''
        else:
            if failure == '':
                if os.path.islink(self.__script__):
                    failure = 'Destination file is a symbol link. This is not allowed.'
                elif not os.path.isfile(self.__script__):
                    failure = 'Destination file does not exist.'
                elif not os.access(self.__script__, os.F_OK):
                    failure = 'Insufficient right to read the destination file.'
    
            if failure:
                failedscript = self.__script__
                print "PHPGATEWAY: Printing error page for: " + failedscript
                cgienv = PHParser.__prepare__(self, error_html, REQUEST)
                cgienv['REDIRECT_SCRIPT_FILENAME'] = failedscript
                cgienv['PHPGATEWAY_ERROR_STRING'] = failure
                cgienv['PHPGATEWAY_SELF'] = '/' + self.absolute_url(1)
            else:
                (filename, fileext) = os.path.splitext(self.__script__)
                if fileext.lower() in PHPEXT:
                    #
                    # *.php or *.php3
                    #
                    print "PHPGATEWAY: About to parse " + self.__script__
                    cgienv = self.__buildenv__(REQUEST)
                    cgienv['PATH_INFO'] = pathinfo
                    print "PHPGATEWAY: Setting CGI ENV PATH_INFO to " + cgienv['PATH_INFO']
                    cgienv['PHPGATEWAY_PATH_INFO'] = '/' + string.join(pathsegs, '/')
                    cgienv['REDIRECT_SCRIPT_FILENAME'] = self.__script__
                    cgienv['DOCUMENT_ROOT'] = self.docroot
                else:
                    #
                    # The request file is not a PHP script. Serve static contents.
                    #
                    (content_type, encoding) = mimetypes.guess_type(self.__script__)
                    print "PHPGATEWAY: Not a PHP script file!"
                    print "PHPGATEWAY: File content type: " + str(content_type)
                    if not content_type:
                        #
                        # ToDo: retrive default content type
                        #
                        content_type = 'application/octet-stream'
                    if encoding:
                        content_type += '; charset=' + encoding
                        print "PHPGATEWAY: File encoding: " + str(encoding)
                    print "PHPGATEWAY: Forging Content-Type header: " + content_type
                    REQUEST.RESPONSE.setHeader('Content-Type', content_type)
                                    
                    filesize = os.path.getsize(self.__script__)
                    if filesize:
                        print "PHPGATEWAY: Forging Content-Length header: " + str(filesize)
                        REQUEST.RESPONSE.setHeader('Content-Length', str(filesize))
                        
                    filemtime = os.path.getmtime(self.__script__)
                    if filemtime:
                        lastmod = strftime("%a, %d %b %Y %H:%M:%S GMT", gmtime(filemtime))
                        print "PHPGATEWAY: Forging Last-Modified header: " + lastmod
                        REQUEST.RESPONSE.setHeader('Last-Modified', lastmod)
                    #
                    # Append the file content
                    #
                    fscript = open(self.__script__, 'rb')
                    self.__result__ = fscript.read()
                    fscript.close()
                #
                # Avoid the script being deleted.
                #
                self.__script__ = ''
                
        self.__pathsegs__ = []
        return cgienv

    # Hooking the __bobo_traverse__ method to allow PHParser responses
    # requests with path information following the script. 

    # i.e. If a PHParser object is added to the root folder as phparser_test
    # /phparser_test/bogus.php will be also served by the same script.

    def __bobo_traverse__(self, TraversalRequest, name):

        """ make /manage and /manage_xxx urls work"""
        if string.find(name, 'manage_') == 0 or name == "manage":
            return getattr(self, name)

        """ record path segments """
        print "PHPGATEWAY: " + self.getId() + ".__bobo_traverse__ " + name

        """ to allow SiteAccess product work """
        #if hasattr(self, name):
        #    return getattr(self, name)
        #
        # 2004-04-21: Wei He <hewei@ied.org.cn>
        # The above two lines didn't work as expected but caused wrong
        # effects.
        #
        # Actually, 'Virtual Host Monster' of the SiteAccess product
        # can work in PHPGateway without the above two lines in 
        # the so called 'simple virtual hosting in a bare Zope server' mode.
        #
        # PHPGateway does not support 'Virtual Host Monster' in the
        # 'apache rewrite' mode and nor have I a plan to so
        # since in that case you can always use apache instead of 
        # PHPGateway to publish the same contents.
        #
        # However, if you have a particular case that need PHPGateway
        # to support, I would be interested to know whether I can make
        # it work.

        if not hasattr(self, '__pathsegs__'):
            self.__pathsegs__ = [name,]
        else:
            self.__pathsegs__.append(name)
        print "PHPGATEWAY: self.__pathsegs__ set to " + str(self.__pathsegs__)

        self.__realname__ = name
        print "PHPGATEWAY: self.__realname__ set to " + self.__realname__

        return self

InitializeClass(PHPGateway)

